/* Written by Shevek as glue between cpplib and Text::CPP */

#include "config.h"
#include "system.h"
#include "cpplib.h"
#include "cpphash.h"

void
TCPP_do_diagnostic(cpp_reader *pfile, int code, int print_dir)
{
}
